@@ -13,7 +13,6 @@ import sys
 import pytest
 
 import bugzilla._cli
-from bugzilla._compatimports import IS_PY3
 
 import tests
 
@@ -30,26 +29,14 @@ def tests_path(filename):
     return filename
 
 
-def fake_stream(text):
-    if IS_PY3:
-        return io.StringIO(text)
-    else:
-        return io.BytesIO(text)
-
-
 def monkeypatch_getpass(monkeypatch):
-    # pylint: disable=undefined-variable
-    if IS_PY3:
-        use_input = input  # noqa
-    else:
-        use_input = raw_input  # noqa
-    monkeypatch.setattr(getpass, "getpass", use_input)
+    monkeypatch.setattr(getpass, "getpass", input)
 
 
 def sanitize_json(rawout):
     # py2.7 leaves trailing whitespace after commas. strip it so
     # tests pass on both python versions
-    return "\n".join([l.rstrip() for l in rawout.splitlines()])
+    return "\n".join([line.rstrip() for line in rawout.splitlines()])
 
 
 def open_functional_bz(bzclass, url, kwargs):
@@ -60,16 +47,8 @@ def open_functional_bz(bzclass, url, kwargs):
     if kwargs.get("force_xmlrpc", False):
         assert bz.is_xmlrpc() is True
 
-    # Set a session timeout of 30 seconds
-    session = bz.get_requests_session()
-    origrequest = session.request
-
-    def fake_request(*args, **kwargs):
-        if "timeout" not in kwargs:
-            kwargs["timeout"] = 60
-        return origrequest(*args, **kwargs)
-
-    session.request = fake_request
+    # Set a request timeout of 60 seconds
+    os.environ["PYTHONBUGZILLA_REQUESTS_TIMEOUT"] = "60"
     return bz
 
 
@@ -118,7 +97,7 @@ def do_run_cli(capsys, monkeypatch,
     argv = shlex.split(argvstr)
     monkeypatch.setattr(sys, "argv", argv)
     if stdin:
-        monkeypatch.setattr(sys, "stdin", fake_stream(stdin))
+        monkeypatch.setattr(sys, "stdin", io.StringIO(stdin))
     else:
         monkeypatch.setattr(sys.stdin, "isatty", lambda: True)
 
